--SavedVariables Setup
local DejaView, private = ...

private.defaults = {
    -- If you have any general settings for you addon,
    -- list them here, or just leave the table empty.
}

local loader = CreateFrame("Frame")
	loader:RegisterEvent("ADDON_LOADED")
	loader:SetScript("OnEvent", function(self, addon)
		if addon ~= DejaView then 
			local function initDB(db, defaults)
				if type(db) ~= "table" then db = {} end
				if type(defaults) ~= "table" then return db end
				for k, v in pairs(defaults) do
					if type(v) == "table" then
						db[k] = initDB(db[k], v)
					elseif type(v) ~= type(db[k]) then
						db[k] = v
					end
				end
			return db
			end

		DejaViewDBPC = initDB(DejaViewDBPC, private.defaults)
		private.db = DejaViewDBPC -- add this
		self:UnregisterEvent("ADDON_LOADED")
		-- Don't place any frames here
		end
	end)

DejaView = {};

local _, private = ...
	private.defaults.optpanelDefaults = {
		point = "RIGHT", 
		relativeTo = "UIParent", 
		relativePoint = "RIGHT", 
		xOffset = 0, 
		yOffset = 0,
	}	
	
DejaView.panel = CreateFrame( "Frame", "DejaViewPanel", UIParent );
-- Register in the Interface Addon Options GUI
-- Set the name for the Category for the Options Panel
DejaView.panel.name = "DejaView";
-- Add the panel to the Interface Options
InterfaceOptions_AddCategory(DejaView.panel);

-- Make a child panel
-- DejaView.childpanel = CreateFrame( "Frame", "DejaViewChild", DejaView.panel);
-- DejaView.childpanel.name = "MyChild";
-- Specify childness of this panel (this puts it under the little red [+], instead of giving it a normal AddOn category)
-- DejaView.childpanel.parent = DejaView.panel.name;
-- Add the child to the Interface Options
-- InterfaceOptions_AddCategory(DejaView.childpanel);

--Panel Title
local dvtitle=CreateFrame("Frame", "dvtitle", DejaViewPanel)
	dvtitle:SetPoint("TOPLEFT", 5, -5)
	dvtitle:SetScale(2.0)
	dvtitle:SetWidth(150)
	dvtitle:SetHeight(50)
	dvtitle:Hide()

local dvtitleFS = dvtitle:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	dvtitleFS:SetText('|cff00c0ffDejaView|r')
	dvtitleFS:SetPoint("TOPLEFT", 0, 0)
	dvtitleFS:SetFont("Fonts\\FRIZQT__.TTF", 10)
	
dvresetcheck = CreateFrame("Button", "DVResetButton", DejaViewPanel, "UIPanelButtonTemplate")
	dvresetcheck:ClearAllPoints()
	dvresetcheck:SetPoint("BOTTOMLEFT", 5, 5)
	dvresetcheck:SetScale(1.25)
	dvresetcheck:SetWidth(125)
	dvresetcheck:SetHeight(30)
	_G[dvresetcheck:GetName() .. "Text"]:SetText("Reset to Default")
	dvresetcheck:SetScript("OnClick", function (self, button, down)
 		DejaViewDBPC = private.defaults;
		ReloadUI();
end)

 dvlockcheck = CreateFrame("CheckButton", "DejaViewLockAll", DejaViewPanel, "InterfaceOptionsCheckButtonTemplate")
	dvlockcheck:ClearAllPoints()
	dvlockcheck:SetPoint("TOPLEFT", 25, -50)
	dvlockcheck:SetScale(1.25)
	_G[dvlockcheck:GetName() .. "Text"]:SetText("Lock all frames")
	dvlockcheck.tooltipText = 'Checked locks all DejaView frames. Unchecked unlocks all DejaView frames.' --Creates a tooltip on mouseover.
	dvlockcheck:SetChecked(true)
	
	dvlockcheckframe=CreateFrame("Frame", "dvlockcheckframe", UIParent)
	dvlockcheckframe:SetClampedToScreen(1)
	dvlockcheckframe:SetFrameStrata(BACKGROUND)
	dvlockcheckframe:SetFrameLevel("1")
	dvlockcheckframe:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT")
	dvlockcheckframe:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT")
	dvlockcheckframe:Hide()
	
	-- Lock/Unlock Frames
	dvlockcheck:SetScript("OnClick", function(self,button,down) 
		if self:GetChecked(true) then
			dvlockcheckframe:Hide()
		else
			dvlockcheckframe:Show()
		end
	end)
	
--Open Categaories Fix
do
	local function get_panel_name(panel)
		local tp = type(panel)
		local cat = INTERFACEOPTIONS_ADDONCATEGORIES
		if tp == "string" then
			for i = 1, #cat do
				local p = cat[i]
				if p.name == panel then
					if p.parent then
						return get_panel_name(p.parent)
					else
						return panel
					end
				end
			end
		elseif tp == "table" then
			for i = 1, #cat do
				local p = cat[i]
				if p == panel then
					if p.parent then
						return get_panel_name(p.parent)
					else
						return panel.name
					end
				end
			end
		end
	end

	local function InterfaceOptionsFrame_OpenToCategory_Fix(panel)
		if doNotRun or InCombatLockdown() then return end
		local panelName = get_panel_name(panel)
		if not panelName then return end -- if its not part of our list return early
		local noncollapsedHeaders = {}
		local shownpanels = 0
		local mypanel
		local t = {}
		local cat = INTERFACEOPTIONS_ADDONCATEGORIES
		for i = 1, #cat do
			local panel = cat[i]
			if not panel.parent or noncollapsedHeaders[panel.parent] then
				if panel.name == panelName then
					panel.collapsed = true
					t.element = panel
					InterfaceOptionsListButton_ToggleSubCategories(t)
					noncollapsedHeaders[panel.name] = true
					mypanel = shownpanels + 1
				end
				if not panel.collapsed then
					noncollapsedHeaders[panel.name] = true
				end
				shownpanels = shownpanels + 1
			end
		end
		local Smin, Smax = InterfaceOptionsFrameAddOnsListScrollBar:GetMinMaxValues()
		if shownpanels > 15 and Smin < Smax then 
		  local val = (Smax/(shownpanels-15))*(mypanel-2)
		  InterfaceOptionsFrameAddOnsListScrollBar:SetValue(val)
		end
		doNotRun = true
		InterfaceOptionsFrame_OpenToCategory(panel)
		doNotRun = false
	end

	hooksecurefunc("InterfaceOptionsFrame_OpenToCategory", InterfaceOptionsFrame_OpenToCategory_Fix)
end

--DV Slash Setup
local RegisteredEvents = {};
local dvslash = CreateFrame("Frame", "DejaViewSlash", UIParent)

dvslash:SetScript("OnEvent", function (self, event, ...) 
	if (RegisteredEvents[event]) then 
	return RegisteredEvents[event](self, event, ...) 
	end
end)

function RegisteredEvents:ADDON_LOADED(event, addon, ...)
	if (addon == "DejaView") then
		SLASH_DEJAVIEW1 = '/dejaview'
		SlashCmdList["DejaView"] = function (msg, editbox)
			DejaView.SlashCmdHandler(msg, editbox)	
		end
		DEFAULT_CHAT_FRAME:AddMessage("DejaView loaded successfully. Type /dejaview for usage",0,192,255)
	end
end

for k, v in pairs(RegisteredEvents) do
	dvslash:RegisterEvent(k)
end

function DejaView.ShowHelp()
	print("DejaView Slash commands (/dejaview):")
	print("  /dejaview lock: Locks all DejaView movable frames.")
	print("  /dejaview unlock: Unlocks all DejaView movable frames.")
	print("  /dejaview config: Open the DejaView addon config menu.")
	print("  /dejaview reset:  Resets DejaView frames to default positions.")
end

function DejaView.SetConfigToDefaults()
	print("Resetting config to defaults")
	DejaViewDBPC = DefaultConfig
	RELOADUI()
end

function DejaView.GetConfigValue(key)
	return DejaViewDBPC[key]
end

function DejaView.PrintPerformanceData()
	UpdateAddOnMemoryUsage()
	local mem = GetAddOnMemoryUsage("DejaView")
	print("DejaView is currently using " .. mem .. " kbytes of memory")
	collectgarbage(collect)
	UpdateAddOnMemoryUsage()
	mem = GetAddOnMemoryUsage("DejaView")
	print("DejaView is currently using " .. mem .. " kbytes of memory after garbage collection")
end

function DejaView.SlashCmdHandler(msg, editbox)
	--print("command is " .. msg .. "\n")
	if (string.lower(msg) == "config") then
		InterfaceOptionsFrame_OpenToCategory("DejaView");
	elseif (string.lower(msg) == "dumpconfig") then
		print("With defaults")
		for k,v in pairs(DVDefaultConfig) do
			print(k,DejaView.GetConfigValue(k))
		end
		print("Direct table")
		for k,v in pairs(DVDefaultConfig) do
			print(k,v)
		end
	elseif (string.lower(msg) == "lock") then
		dvlockcheckframe:Hide()
		dvlockcheck:SetChecked(true)
	elseif (string.lower(msg) == "unlock") then
		dvlockcheckframe:Show()
		dvlockcheck:SetChecked(false)
	elseif (string.lower(msg) == "reset") then
		DejaViewDBPC = private.defaults;
		ReloadUI();
	elseif (string.lower(msg) == "perf") then
		DejaView.PrintPerformanceData()
	else
		DejaView.ShowHelp()
	end
end
	SlashCmdList["DEJAVIEW"] = DejaView.SlashCmdHandler;